/*
	$Id: NewsTransfer.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Gregory Vincic
	Copyright (C) 2007 Jari Häkkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.News;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>News</dd>
<dt><b>Unused columns</b></dt>
<dd>owner</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li>All news items will be owned by root.</li>
</ul>
</dd>
</dl>

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/

public class NewsTransfer extends Transfer
{
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.NewsTransfer");
	
	/**
	Date format for the start date is yyyy-MM-dd HH:mm:ss
	 */
	private SimpleDateFormat df = null;

	NewsTransfer()
	{
		df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	}

	public void start()
	{
		log.info("Transferring News...");
		int count = getInt("countNews");
		initIdMap(count);
		startProgress(count, "News");
		run(prepareStatementFromFile("selectNews"));
		done();
	}

	/**
	Returns a News item from the current row in a given ResultSet.
	
	@param dc {@link DbControl}
	@param rs {@link ResultSet}
	@return News
	 */
	News createItem(DbControl dc, ResultSet rs) throws BaseException,
			SQLException
	{
		Date startDate = null;
		try
		{
			startDate = df.parse(rs.getString(4));
		}
		catch (Exception e)
		{
			log.error("Couldn't parse news:" + rs.getString(2) + " startdate", e);
			stop();
		}
		News news = News.getNew(dc, startDate, startDate);
		news.setName(rs.getString(2));
		news.setDescription(rs.getString(3));
		news.setRemoved(rs.getBoolean(5));
		if (log.isDebugEnabled())
		{
			log.debug("Transfered News: " + news.getName() + "\tBASE2 id: (" + news.getId() + ")");			
		}
		return news;
	}
}
