/*
	$Id: HasAnnotationRestriction.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Query;

import org.hibernate.SQLQuery;

/**
	Restricts a query by the presence of absence of an annotation.
	
	@author Nicklas
	@version 2.7
	@see net.sf.basedb.core.query.Annotations
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class HasAnnotationRestriction
	extends AnnotationRestriction
{

	private boolean hasBeenAnnotatated;
	
	/**
		Create a new restriction that checks if an item has been annotatated
		or not.
		
		@param alias The alias of a joined item where the annotations are
			located or null to use the root entity of the query
		@param annotationType The annotation type to use in the query
		@param hasBeenAnnotatated TRUE to look for items that has been annotated with the given
			annotation type, FALSE to look for items that hasn't
		@param includeInheriting If items inherting the annotation should be returned
			by the query or not
	*/
	public HasAnnotationRestriction(String alias, AnnotationType annotationType, 
		boolean hasBeenAnnotatated, boolean includeInheriting)
	{
		this(alias, annotationType.getId(), hasBeenAnnotatated, includeInheriting);
	}
	
	/**
		@see #HasAnnotationRestriction(String, AnnotationType, boolean, boolean)
	*/
	public HasAnnotationRestriction(String alias, int annotationTypeId, 
		boolean hasBeenAnnotatated, boolean includeInheriting)
	{
		super(alias, annotationTypeId, null, includeInheriting);
		this.hasBeenAnnotatated = hasBeenAnnotatated;
	}

	/*
		From the AnnotationRestriction class
		-------------------------------------------
	*/
	String getRestrictionSql(Query query, DbControl dc)
		throws BaseException
	{
		return null;
	}
	void setRestrictionParameters(SQLQuery query)
	{}
	boolean notAmongSelected()
	{
		return !hasBeenAnnotatated;
	}
	// -----------------------------------------

}

