/*
	$Id: AnnotatableData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	An annotatable item is an item that can be annotated with
	name/value pairs. All annotations are held in an {@link
	AnnotationSetData} object.
	<p>
	This interface defines Hibernate database mapping for the
	<code>annotationSet</code> association to a column with
	the name <code>annotationset_id</code>. If a subclass wants
	to map the association to another column, it should override
	the {@link #getAnnotationSet()} method and add a Hibernate tag 
	in the comment.
	<p>
	
	<b>Reference implementation</b><br>
	<pre class="code">
private AnnotationSetData annotationSet;
public AnnotationSetData getAnnotationSet()
{
   return annotationSet;
}
public void setAnnotationSet(AnnotationSetData annotationSet)
{
   this.annotationSet = annotationSet;
}
</pre>

	@author Nicklas
	@version 2.0
	@see AnnotatedData
	@see AnnotationSetData
	@see AnnotationData
	@see AnnotationTypeData
	@see net.sf.basedb.core.Annotatable
	@see <a href="../../../../../../html/developerdoc/api/data_api.annotations.html">Annotations overview</a>
*/
public interface AnnotatableData
	extends IdentifiableData
{
	/**
		Get the annotation set that holds the annotations for an item.
		@return An <code>AnnotationSetData</code> item or null if no annotations exist
		@hibernate.many-to-one column="`annotationset_id`" not-null="false" unique="true" 
			outer-join="false" cascade="delete"
	*/
	public AnnotationSetData getAnnotationSet();

	/**
		Change the annotation set. Use null to remove the annotations.
	 	@param annotationSet Annotation set to change to. 
	*/
	public void setAnnotationSet(AnnotationSetData annotationSet);
}
