/*
	$Id: QuotaIndex.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.io.Serializable;

/**
	This is a helper class for the {@link QuotaData} item to
	help with the Hibernate mapping between quota and
	quota type/location/max bytes. This class holds quota type/location
	pairs used as keys in the {@link QuotaData#getQuotaValues()}
	map.

	@author enell
	@version 2.0
	@see <a href="../../../../../../html/developerdoc/api/data_api.quota.html">Quota overview</a>
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
@SuppressWarnings({"serial"})
public class QuotaIndex
	implements Serializable
{
	QuotaIndex()
	{}
	
	public QuotaIndex(QuotaTypeData quotaType, int location)
	{
		this.quotaType = quotaType;
		this.location = location;
	}

	private int location;

	/**
	 	@hibernate.property column="`location`" type="int"
	*/
	public int getLocation()
	{
		return location;
	}

	void setLocation(int location)
	{
		this.location = location;
	}

	private transient QuotaTypeData quotaType;

	/**
		@hibernate.many-to-one column="`quotaType_id`" outer-join="false"
	*/
	public QuotaTypeData getQuotaType()
	{
		return quotaType;
	}

	void setQuotaType(QuotaTypeData quotaType)
	{
		this.quotaType = quotaType;
	}

	/**
		Check if this object is equal to another <code>QuotaIndex</code>
		object. They are equal if both have the same quotatype id and location.
	*/
	public boolean equals(Object o)
	{
		if ((o == null) || (getClass() != o.getClass()))
		{
			return false;
		}
		QuotaIndex qi = (QuotaIndex) o;
		return (this.quotaType.getId() == quotaType.getId()) && (this.location == qi.location);
	}

	/**
		Calculate the hash code for the object.
	*/
	public int hashCode()
	{
		return (int) (53 * quotaType.getId() + 17 * location);
	}
}
