/**
	$Id: PowerExpression.java 4912 2009-04-29 11:49:20Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Arrays;
import java.util.Collection;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HibernateUtil;

/**
	Calculate the power: base ^ exponent

	@author Nicklas
	@version 2.12
	@see Expressions#power(Expression, Expression)
	@base.modified $Date: 2009-04-29 13:49:20 +0200 (Wed, 29 Apr 2009) $
*/
public class PowerExpression
	implements Expression
{
	private final Expression base;
	private final Expression exponent;

	PowerExpression(Expression base, Expression exponent)
	{
		assert base != null : "base == null";
		assert exponent != null : "exponent == null";
		this.base = base;
		this.exponent = exponent;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return HibernateUtil.getDbEngine().power(base.toQl(query, dc), exponent.toQl(query, dc));
	}
	/**
		@return The expression to exponentiate
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Arrays.asList(base, exponent);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "POWER(" + base + "," + exponent + ")";
	}
	// -------------------------------------------
}