/**
	$Id: ProtocolParameterLoader.java 4749 2009-02-11 08:49:39Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultIterator;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.node.NameableNameGenerator;
import net.sf.basedb.util.overview.node.NodeFactory;

/**
	Node loader implementation for protocol parameters. Protocol
	parameters are loaded as property-type child nodes of
	a protocol. The protocol must have a parent node with
	an item that is {@link Annotatable} since this is were the
	parameter values are stored.
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-11 09:49:39 +0100 (Wed, 11 Feb 2009) $
*/
public class ProtocolParameterLoader
	extends BasicItemNodeLoader<AnnotationType>
{

	public ProtocolParameterLoader()
	{
		super("PROTOCOL.PARAMETER", DENY_ROOT_NODE, 
				new NameableNameGenerator<AnnotationType>("parameter", null));
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	@Override
	public Node createPropertyNode(DbControl dc, OverviewContext context, Node protocolNode)
	{
		NodeFactory<AnnotationType> nf = getNodeFactory(dc, context);
		Node returnNode = null;
		
		// Load all protocol parameters for the given protocol
		Protocol protocol = (Protocol)protocolNode.getItem(dc);
		ItemQuery<AnnotationType> query = context.initQuery(protocol.getParameters(), "name");
		ItemResultIterator<AnnotationType> it = query.iterate(dc);
		while (it.hasNext())
		{
			AnnotationType parameter = it.next();
			returnNode = createItemNode(nf, parameter, null, false, protocolNode, ChildNodeDirection.NONE);
		}
		postValidateFolder(nf, null, protocolNode, false);
		return protocolNode.numChildren() > 1 ? null : returnNode;
	}
	// -----------------------------------
}