/*
 $Id$

 Copyright (C) 2007 Johan Enell

 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.

 BASE is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileStoreUtil;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataBatcher;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Reporter;
import net.sf.basedb.core.ReporterBatcher;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.ConsoleProgressReporter;
import net.sf.basedb.util.basefile.BaseFileWriter;
import net.sf.basedb.util.export.spotdata.BaseFileExporter;
import net.sf.basedb.util.export.spotdata.DynamicField;
import net.sf.basedb.util.export.spotdata.ExportableFieldFactory;
import net.sf.basedb.util.export.spotdata.MatrixBaseFileExporter;
import net.sf.basedb.util.export.spotdata.SerialBaseFileExporter;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.FlatFileParser.Line;

import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

/**

 @author johan, nicklas
 @version 2.3
 @base.modified $Date$
 */
public class TestBioAsssaySetExporter
{
	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing BioAsssaySetExporter");
		
		int r1 = test_createReporter("exportreporter1");
		int r2 = test_createReporter("exportreporter2");
		
		int rba1 = 0;
		int rba2 = 0;
		int annotationType = test_createAnnotationType("testExportAnnotation");
		if (ok)
		{
			rba1 = test_createRawBioAssay("export1", annotationType, "1");
			rba2 = test_createRawBioAssay("export2", annotationType, "2");
			test_addRawData(rba1, r1, r2, r1);
			test_addRawData(rba2, r1, r2, r2);
		}

		int experiment = 0;
		int fileSerialNoMerge = TestFile.test_create("serial.nomerge.txt", false, false);
		int fileSerialMerge = TestFile.test_create("serial.merge.txt", false, false);
		int fileMatrixNoMerge = TestFile.test_create("matrix.nomerge.txt", false, false);
		int fileMatrixMerge = TestFile.test_create("matrix.merge.txt", false, false);
		int bas = 0;
		if (ok)
		{
			experiment = TestExperiment.test_create("genepix", false);
			test_addExperimentalFactor(experiment, annotationType);
			TestExperiment.test_add_rawbioassay(experiment, rba1);
			TestExperiment.test_add_rawbioassay(experiment, rba2);
			bas = TestExperiment.test_create_root_bioassayset_using_calculator(experiment, "medianfg", new int[] {rba1, rba2});
			
			test_exportBaseFile(bas, fileSerialNoMerge, false, false, true);
			test_exportBaseFile(bas, fileSerialMerge, false, true, true);
			test_exportBaseFile(bas, fileMatrixNoMerge, true, false, true);
			test_exportBaseFile(bas, fileMatrixMerge, true, true, true);
		}
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		
		TestExperiment.test_delete(experiment);
		TestRawBioAssay.test_delete(rba1);
		TestRawBioAssay.test_delete(rba2);
		TestFile.test_delete(fileSerialNoMerge);
		TestFile.test_delete(fileSerialMerge);
		TestFile.test_delete(fileMatrixNoMerge);
		TestFile.test_delete(fileMatrixMerge);
		TestAnnotationType.test_delete(annotationType);
		test_deleteReporter(r1, r2);
		
		write("++Testing BioAsssaySetExporter "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	private static int test_addExperimentalFactor(int experimentId, int annotationTypeId)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Experiment e = Experiment.getById(dc, experimentId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			e.addExperimentalFactor(at);
			dc.commit();
			write("--Experimental factor was added to experiment.");
		}
		catch (Throwable ex)
		{
			write("--Add experimental factor FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	private static void test_addRawData(int rbaId, int r1Id, int r2Id, int r3Id)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			RawBioAssay rba = RawBioAssay.getById(dc, rbaId);
			ReporterData r1 = Reporter.getById(dc, r1Id);
			ReporterData r2 = Reporter.getById(dc, r2Id);
			ReporterData r3 = Reporter.getById(dc, r3Id);
			RawDataBatcher rdb = rba.getRawDataBatcher(null, null);
			for (int i=0; i < 4; ++i)
			{
				RawData rd = rdb.newRawData();
				rd.setBlock(1);
				rd.setMetaGridX(0);
				rd.setMetaGridY(0);
				rd.setColumn(i%2);
				rd.setRow((int)Math.floor(i/2));
				rd.setReporter(i < 2 ? r1 : r2);
				rd.setExtended("flags", Integer.valueOf(0));
				rd.setExtended("ch1FgMedian", Float.valueOf(1));
				rd.setExtended("ch2FgMedian", Float.valueOf(2));
				rdb.insert(rd);
			}
			
			// the last position
			RawData rd = rdb.newRawData();
			rd.setBlock(1);
			rd.setMetaGridX(0);
			rd.setMetaGridY(0);
			rd.setColumn(0);
			rd.setRow(2);
			rd.setReporter(r3);
			rd.setExtended("flags", Integer.valueOf(-50));
			rd.setExtended("ch1FgMedian", Float.valueOf(10));
			rd.setExtended("ch2FgMedian", Float.valueOf(20));
			rdb.insert(rd);
			
			rdb.close();
			dc.commit();
			write("--RawData was added to RawBioAssay "+rba.toString());
		}
		catch (Throwable ex)
		{
			write("--Add rawdata FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	private static int test_createAnnotationType(String name)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			AnnotationType at = AnnotationType.getNew(dc, Type.STRING);
			at.setName(name);
			at.enableForItem(Item.RAWBIOASSAY);
			dc.saveItem(at);
			dc.commit();
			id = at.getId();
			write("--AnnotationType created");
		}
		catch (Throwable ex)
		{
			write("--Create AnnotationType FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	private static int test_createRawBioAssay(String name, int annotationTypeId, String annotationValue)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Platform generic = Platform.getByExternalId(dc, Platform.GENERIC);
			RawBioAssay rba = RawBioAssay.getNew(dc, generic, RawDataTypes.getSafeRawDataType("genepix"));
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			
			rba.setName(name);
			dc.saveItem(rba);
			
			AnnotationSet as = rba.getAnnotationSet();
			Annotation a = as.getAnnotation(at);
			
			a.setValue(annotationValue);
			
			dc.commit();
			id = rba.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(rba, false);
			write("--Create raw bioassay OK");
		}
		catch (Throwable ex)
		{
			write("--Create raw bioassay FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	private static int test_createReporter(String name)
	{
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterBatcher rb = ReporterBatcher.getNew(dc);
			ReporterData r = Reporter.getNew(name);
			r.setName(name);
			rb.insert(r);
			rb.close();
			dc.commit();
			
			dc = TestUtil.getDbControl();
			r = Reporter.getByExternalId(dc, name);
			id = r.getId();
			write("--Created reporter "+name+" with id("+id+")");
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	private static void test_deleteReporter(int... ids)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ReporterBatcher rb = ReporterBatcher.getNew(dc);
			for (int r : ids)
			{
				if (r > -1) rb.delete(r);
			}
			rb.close();
			dc.commit();
			write("--Deleted reporters "+Arrays.toString(ids));
		}
		catch (Throwable ex)
		{
			write("--Deleted reporters "+Arrays.toString(ids) +" FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	/**
		The BioAssaySet thats beeing tested must be the one created by this class.
	 */
	static void test_exportBaseFile(int bioAssaySetId, int fileId, 
			boolean matrix, boolean mergeReporters, boolean verify)
	{
		DbControl dc = null;
		FlatFileParser ffp = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioAssaySet bas = BioAssaySet.getById(dc, bioAssaySetId);
			File file = File.getById(dc, fileId);
			ItemQuery<BioAssay> query = bas.getBioAssays();
			query.order(Orders.asc(Hql.property("name")));
			query.order(Orders.asc(Hql.property("id")));
			List<BioAssay> assays = new ArrayList<BioAssay>(query.list(dc));
			
			// Initialise exporter
			BaseFileExporter bfe = matrix ? new MatrixBaseFileExporter() : new SerialBaseFileExporter();
			bfe.setDbControl(dc);
			bfe.setSource(bas);
			bfe.setAverageOnReporter(mergeReporters);
			if (!TestUtil.getSilent()) bfe.setProgressReporter(new ConsoleProgressReporter());
			bfe.setWriter(new BaseFileWriter(new OutputStreamWriter(file.getUploadStream(false))));
			
			// Parameters that go in the 'settings' section
			bfe.setParameter("b", "some parameter");
			bfe.setParameter("a", "some other parameter");

			// Configure the reporter fields to export
			List<DynamicField> reporterFields = new ArrayList<DynamicField>();
			reporterFields.add(ExportableFieldFactory.reporter("id", null, Formula.AverageMethod.NONE, null));
			reporterFields.add(ExportableFieldFactory.reporter("name", null, Formula.AverageMethod.NONE, null));
			reporterFields.add(ExportableFieldFactory.simple(Dynamic.column(VirtualColumn.POSITION), "position", Formula.AverageMethod.NONE, null));
			bfe.addReporterFields(reporterFields);

			// Configure the spot fields to export
			ItemQuery<Formula> ratio = Formula.getQuery(Formula.Type.COLUMN_EXPRESSION, 
					bas.getRawDataType(), bas.getIntensityTransform());
				ratio.restrict(Restrictions.eq(Hql.property("name"), Expressions.string("Ratio, ch1 / ch2")));
				if (ratio.count(dc) < 1)
				{
					throw new Exception("Cant find formula 'Ratio, ch1 / ch2'");
				}
			
			List<DynamicField> spotFields = new ArrayList<DynamicField>();
			spotFields.add(ExportableFieldFactory.channel(1, "intensity1", null));
			spotFields.add(ExportableFieldFactory.rawData("row", null, Formula.AverageMethod.NONE, null));
			spotFields.add(ExportableFieldFactory.rawData("flags", null, Formula.AverageMethod.NONE, null));
			spotFields.add(ExportableFieldFactory.formula(ratio.list(dc).get(0), null, null));
			bfe.addSpotFields(spotFields);
			
			bfe.doExport();
			FileStoreUtil.setDataFile(dc, bas, 
					matrix ? DataFileType.BASEFILE_MATRIX : DataFileType.BASEFILE_SERIAL, 
					file);
			if (!TestUtil.getSilent()) write("");
			dc.commit();
			
			// Now it is time to test if the result is what we expected
			if (verify)
			{
				dc = TestUtil.getDbControl();
				ffp = new FlatFileParser();
				ffp.setSectionRegexp(Pattern.compile("section[\\t](.*)"));
				ffp.setHeaderRegexp(Pattern.compile("(.*?)\\t(.*)"));
				ffp.setDataHeaderRegexp(Pattern.compile("%"));
				ffp.setDataFooterRegexp(Pattern.compile("^$"));
				ffp.setDataSplitterRegexp(Pattern.compile("\\t"));
				ffp.setInputStream(file.getDownloadStream(0), Config.getCharset());
				ffp.setUseNullIfEmpty(false);
	
				List<Pattern[]> rowPatterns;
				
				// Test BASEfile marker
				ffp.parseHeaders();
				TestFlatFileParser.testLine(ffp.getLine(0), "BASEfile");
	
				// Test settings section
				TestFlatFileParser.testSection(ffp.nextSection(), "settings");
				ffp.parseHeaders();
				TestFlatFileParser.testHeaderLine(ffp.getLine(0), "b", "some parameter");
				TestFlatFileParser.testHeaderLine(ffp.getLine(1), "a", "some other parameter");
	
				// Test assays section
				Line section = ffp.nextSection();
				TestFlatFileParser.testSection(section, "assays");
				ffp.parseHeaders();
				TestFlatFileParser.testHeaderLine(ffp.getLine(0), "columns", "\\t", "id", "name", "testExportAnnotation");
				TestFlatFileParser.testHeaderLine(ffp.getLine(1), "annotationColumns", "\\t", "testExportAnnotation");
				TestFlatFileParser.testHeaderLine(ffp.getLine(2), "count", "2");
				rowPatterns = new ArrayList<Pattern[]>(2);
				rowPatterns.add(null);
				rowPatterns.add(null);
				TestFlatFileParser.testDataSection(ffp, rowPatterns, section);
				
				// Test spot section(s)
				int numOfSections = matrix ? 1 : (int) bas.getBioAssays().count(dc);
				String spotsCount;
				if (matrix)
				{
					spotsCount = mergeReporters ? "2" : "6";
				}
				else
				{
					spotsCount = mergeReporters ? "2" : "5";
				}
				
				rowPatterns.clear();
				if (mergeReporters)
				{
					Pattern[] p = new Pattern[matrix ? 11 : 7];
					p[0] = Pattern.compile("\\d+"); // Reporter id
					p[1] = Pattern.compile("exportreporter\\d"); // Reporter name
					p[2] = Pattern.compile("[123456]");	// Position
					p[3] = Pattern.compile("[14](\\.0)?"); // spot intensity
					p[4] = Pattern.compile("");	// row
					p[5] = p[4];						// flags
					p[6] = Pattern.compile("0\\.5"); // Ratio
					if (matrix)
					{
						for (int i = 7; i < 11; ++i) // Repeat for second assay
						{
							p[i] = p[i-4];
						}
					}
					rowPatterns.add(p);
					rowPatterns.add(p);
				}
				else
				{
					if (matrix)
					{
						Pattern[] p1 = new Pattern[11];
						p1[0] = Pattern.compile("\\d+"); // Reporter id
						p1[1] = Pattern.compile("exportreporter\\d"); // Reporter name
						p1[2] = Pattern.compile("[1234]");	// Position
						p1[3] = Pattern.compile("1?(\\.0)?");  // Spot intensity
						p1[4] = Pattern.compile("[01]");	// Row
						p1[5] = Pattern.compile("0");	// Flags
						p1[6] = Pattern.compile("0\\.5");  // Ratio
						
						Pattern[] p2 = new Pattern[11];
						p2[0] = p1[0];
						p2[1] = p1[1];
						p2[2] = Pattern.compile("[56]");	// Position
						p2[3] = Pattern.compile("(10(\\.0)?)|");  // Spot intensity
						p2[4] = Pattern.compile("2|");	// Row
						p2[5] = Pattern.compile("-50|");	// Flags
						p2[6] = Pattern.compile("0\\.5|"); //Ratio
						for (int i = 7; i < 11; ++i)
						{
							p1[i] = p1[i-4];
							p2[i] = p2[i-4];
						}
						rowPatterns.add(p1);
						rowPatterns.add(p1);
						rowPatterns.add(p1);
						rowPatterns.add(p1);
						rowPatterns.add(p2);
						rowPatterns.add(p2);
					}
					else
					{
						Pattern[] p = new Pattern[7];
						p[0] = Pattern.compile("\\d+"); // Reporter id
						p[1] = Pattern.compile("exportreporter\\d"); // Reporter name
						p[2] = Pattern.compile("[123456]");	// Position
						p[3] = Pattern.compile("10?(\\.0)?");  // Spot intensity
						p[4] = Pattern.compile("[012]");	// Row
						p[5] = Pattern.compile("(0)|(-50)");	// Flags
						p[6] = Pattern.compile("0\\.5");  // Ratio
						rowPatterns.add(p);
						rowPatterns.add(p);
						rowPatterns.add(p);
						rowPatterns.add(p);
						rowPatterns.add(p);
					}
				}
	
				section = ffp.nextSection();
				int sectionIndex = 0;
				while (section != null)
				{
					TestFlatFileParser.testSection(section, "spots");
					ffp.parseHeaders();
					List<String> expectedAssays = new ArrayList<String>();
					if (matrix)
					{
						for (BioAssay ba : assays)
						{
							expectedAssays.add(Integer.toString(ba.getId()));
						}
					}
					else
					{
						expectedAssays.add(Integer.toString(assays.get(sectionIndex).getId()));
					}
					TestFlatFileParser.testHeaderLine(ffp.getLine(0), "channels", "2");
					TestFlatFileParser.testHeaderLine(ffp.getLine(1), "assays", "\\t", expectedAssays);
					TestFlatFileParser.testHeaderLine(ffp.getLine(2), "columns", "\\t", "id", "name", "position", "assayData");
					TestFlatFileParser.testHeaderLine(ffp.getLine(3), "assayFields", "\\t", "intensity1", "row", "flags", "Ratio, ch1 / ch2");
					TestFlatFileParser.testHeaderLine(ffp.getLine(4), "count", spotsCount);
					TestFlatFileParser.testDataSection(ffp, rowPatterns, section);
					
					section = ffp.nextSection();
					sectionIndex++;
				}
			}
			write("--Exporting basefile [matrix:"+matrix+", mergeReporters:"+mergeReporters+"] SUCCEEDED");
		}
		catch (Throwable ex)
		{
			write("--Exporting basefile [matrix:"+matrix+", mergeReporters:"+mergeReporters+"] FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
}
