/*
	$Id: TestMessage.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import java.util.Date;

public class TestMessage
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing messages");
		write_header();
		// Standard tests: create, load, list
		int toUserId = TestUser.test_create(true);
		int id = test_create(toUserId, "Test program", true);
		int id2 = test_create(toUserId, "Test program", false);
		test_load(id);
		test_list(toUserId, 2);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		TestUser.test_delete(toUserId);
		write("++Testing messages "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int toUserId, String from, boolean setAll)
	{
		if (toUserId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.MESSAGE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User user = User.getById(dc, toUserId);
			Message m = Message.getNew(dc, user, from, null, null);
			if (setAll)
			{
				m.setName("Test message");
				m.setDescription("Added at "+new Date());
			}
			dc.saveItem(m);
			dc.commit();
			id = m.getId();
			write_item(0, m);
			write("--Create message OK");
		}
		catch (Throwable ex)
		{
			write("--Create message FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Message m = Message.getById(dc, id);
			write_item(0, m);
			write("--Load message OK");
		}
		catch (Throwable ex)
		{
			write("--Load message FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int toUserId, int expectedResults)
	{
		if (toUserId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Message> l = Message.getQuery(User.getById(dc, toUserId)).list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List messages OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List messages FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Message m = Message.getById(dc, id);
			dc.deleteItem(m);
			dc.commit();
			write("--Delete message OK");
		}
		catch (Throwable ex)
		{
			write("--Delete message FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tSubject   \tMessage\tFrom\tTime sent");
			write("-- \t-- \t--------- \t-------\t----\t---------");
		}
	}
	static void write_item(int i, Message m)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+m.getId()+"\t"+m.getName()+"\t"+m.getDescription()+"\t"+
			m.getFrom()+"\t"+m.getTimeSent());
	}
	static void write(String message)
	{
		System.out.println(message);
	}


}

