/*
	$Id: TestPlateGeometry.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;

public class TestPlateGeometry
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing plate geometries");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(5, 10, true);
		int id2 = test_create(6, 12, false);
		test_load(id);
		test_list(-1);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		write("++Testing plate geometries "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int rows, int columns, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.PLATEGEOMETRY)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PlateGeometry pg = PlateGeometry.getNew(dc, rows, columns);
			if (setAll)
			{
				pg.setName("Test plate geometry");
				pg.setDescription("Added at "+new Date());
			}
			dc.saveItem(pg);
			dc.commit();
			id = pg.getId();
			write_item(0, pg);
			write("--Create plate geometry OK");
		}
		catch (Throwable ex)
		{
			write("--Create plate geometry FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PlateGeometry pg = PlateGeometry.getById(dc, id);
			write_item(0, pg);
			write("--Load plate geometry OK");
		}
		catch (Throwable ex)
		{
			write("--Load plate geometry FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<PlateGeometry> l = PlateGeometry.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List plate geometries OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List plate geometries FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PlateGeometry pg = PlateGeometry.getById(dc, id);
			dc.deleteItem(pg);
			dc.commit();
			write("--Delete plate geometry OK");
		}
		catch (Throwable ex)
		{
			write("--Delete plate geometry FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			System.out.println("   \tID \tName      \tDescription\tRows\tColumns");
			System.out.println("-- \t-- \t--------- \t-----------\t----\t-------");
		}
	}
	static void write_item(int i, PlateGeometry pg)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+pg.getId()+"\t"+pg.getName()+"\t"+pg.getDescription()+
			"\t"+pg.getRows()+"\t"+pg.getColumns());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

}
