import java.io.FileInputStream;
import java.util.Date;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Location;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.Quota;
import net.sf.basedb.core.QuotaException;
import net.sf.basedb.core.QuotaType;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.User;

/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 @author enell
 
 */
public class TestQuota
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}
	
	static boolean test_all()
	{
		write("++Testing quota");
		write_header();

		// Standard tests: create, load, list
		int id1 = test_create(true);
		int id2 = test_create(false);
		test_load(id1);
		test_list(-1);

		// Extra tests: create user, modify quota, consume and release quota
		int user_id = TestUser.test_create(false);
		test_assign_quota(user_id, id1);
		test_show_usage(user_id);
		int consume_id1 = test_consume(user_id, "data/test.upload.txt", false);
		test_show_usage(user_id);
		int consume_id2 = test_consume(user_id, "data/test.upload.png", true);
		test_show_usage(user_id);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_release(consume_id1);
		test_release(consume_id2);
		test_show_usage(user_id);
		TestUser.test_delete(user_id);

		// Standard test: Delete
		test_delete(id1);
		test_delete(id2);
		write("++Testing quota "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	private static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Quota> l = Quota.getQuery().list(dc);
			for (int i = 0; i < l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected " + expectedResults + " results, not " + l.size());
			}
			write("--List quotas OK (" + l.size() + ")");
		}
		catch (Throwable ex)
		{
			write("--List quotas FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Quota q = Quota.getById(dc, id);
			write_item(0, q);
			write("--Load quota OK");
		}
		catch (Throwable ex)
		{
			write("--Load quota FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.QUOTA)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Quota q = Quota.getNew(dc, 200000L);
			if(setAll)
			{
				QuotaType total = QuotaType.getById(dc, SystemItems.getId(QuotaType.TOTAL));
				QuotaType file = QuotaType.getById(dc, SystemItems.getId(QuotaType.FILE));
				
				q.setName("Test quota");
				q.setDescription("Added at "+new Date());
				q.setQuotaValue(total, Location.PRIMARY, 30000);
				q.setQuotaValue(file, Location.PRIMARY, 10000);
			}
			dc.saveItem(q);
			dc.commit();
			id = q.getId();
			write_item(0, q);
			write("--Create quota OK");
		}
		catch (Throwable ex)
		{
			write("--Create quota FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Quota q = Quota.getById(dc, id);
			dc.deleteItem(q);
			dc.commit();
			write("--Delete quota OK");
		}
		catch (Throwable ex)
		{
			write("--Delete quota FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_assign_quota(int userId, int quotaId)
	{
		if (userId == 0 || quotaId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Quota q = Quota.getById(dc, quotaId);
			User u = User.getById(dc, userId);
			u.setQuota(q);
			dc.commit();
			write("--Assign quota OK");
		}
		catch (Throwable ex)
		{
			write("--Assign quota FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static int test_consume(int userId, String filename, boolean shouldFail)
	{
		int id = 0;
		DbControl dc = null;
		File f = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, userId == 0 ? dc.getSessionControl().getLoggedInUserId() : userId);
			f = File.getNew(dc, Directory.getById(dc, SystemItems.getId(Directory.ROOT)));
			f.setName(filename.replace("/", "_"));
			f.setOwner(u);
			dc.saveItem(f);
			f.upload(new FileInputStream(filename), false);
			dc.commit();
			id = f.getId();
			if (shouldFail)
			{
				write("--Consume quota FAILED (Expected QuotaException)");
				ok = false;
			}
			else
			{
				write("--Consume quota OK");
			}
		}
		catch (QuotaException ex)
		{
			if (shouldFail)
			{
				write("--Consume quota OK ("+ex.getMessage()+")");
			}
			else
			{
				write("--Consume quota FAILED");
				ex.printStackTrace();
				ok = false;
			}
		}
		catch (Throwable ex)
		{
			write("--Consume quota FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_release(int fileId)
	{
		if (fileId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			File f = File.getById(dc, fileId);
			dc.deleteItem(f);
			dc.commit();
			write("--Release quota OK");
		}
		catch (Throwable ex)
		{
			write("--Release quota FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_show_usage(int userId)
	{
		if (userId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, userId);
			write_item(0, u);
			dc.commit();
			write("--Show disc usage OK");
		}
		catch (Throwable ex)
		{
			write("--Show disc usage FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	//	static write methodes
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID\tName\tTotal\tFile");
			write("-- \t--\t----\t-----\t----");
		}
	}
	
	static void write_item(int i, Quota q)
		throws BaseException
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			if (!TestUtil.getSilent())
			{
				QuotaType total = QuotaType.getById(dc, SystemItems.getId(QuotaType.TOTAL));
				QuotaType file = QuotaType.getById(dc, SystemItems.getId(QuotaType.FILE));
			
				write(i+":\t"+q.getId()+"\t"+q.getName()+
					"\t"+q.getQuotaValue(total, Location.PRIMARY)+
					"\t"+q.getQuotaValue(file, Location.PRIMARY));
			}
		}
		catch (Throwable ex)
		{
			write("--write_item quota FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write_item(int i, User u)
		throws BaseException
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			if (!TestUtil.getSilent())
			{
				QuotaType total = QuotaType.getById(dc, SystemItems.getId(QuotaType.TOTAL));
				QuotaType file = QuotaType.getById(dc, SystemItems.getId(QuotaType.FILE));
			
				write(i+":\t"+u.getId()+"\t"+u.getName()+
					"\tTotal: "+u.getDiskUsage(total, Location.PRIMARY)+
					"\tFile: "+u.getDiskUsage(file, Location.PRIMARY)
				);
			}
		}
		catch (Throwable ex)
		{
			write("--write_item user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
