import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.ReporterType;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Response;

/*
	$Id: TestReporterFlatFileImporter.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

public class TestReporterFlatFileImporter
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing reporter import using plugin");
		// Create reporter type and upload file
		int reporterTypeId = TestReporterType.test_create();
		int fileId = TestFile.test_create("data/test.reporter.import.txt", false, false);
		
		// Create plugin configuration and job
		int pluginDefinitionId  = TestPluginDefinition.test_get("net.sf.basedb.plugins.ReporterFlatFileImporter");
		int pluginConfigurationId = test_create_configuration(pluginDefinitionId, reporterTypeId);
		TestPluginConfiguration.test_load(pluginConfigurationId);
		int jobId = test_create_job(pluginConfigurationId, fileId, "update");
		
		// Execute job
		ok &= TestJob.test_execute(jobId);
		
		// Test: list the result
		TestReporter.write_header();
		TestReporter.test_list(-1);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		int jobId2 = test_create_job(pluginConfigurationId, fileId, "delete");
		ok &= TestJob.test_execute(jobId2);
		
//		TestReporter.test_delete();

		TestJob.test_delete(jobId);
		TestJob.test_delete(jobId2);
		TestPluginConfiguration.test_delete(pluginConfigurationId);
		TestFile.test_delete(fileId);
		TestReporterType.test_delete(reporterTypeId);

		write("++Testing reporter import using plugin "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create_configuration(int pluginDefinitionId, int reporterTypeId)
	{
		if (pluginDefinitionId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLUGINCONFIGURATION)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginDefinition pd = PluginDefinition.getById(dc, pluginDefinitionId);
			PluginConfiguration pc = pd.newPluginConfiguration();
			pc.setName(pd.getName());
			
			PluginConfigurationRequest request = pc.configure();

			write_request_information(request.getRequestInformation());
			
			request.setParameterValue("dataSplitterRegexp", "\\t");
			request.setParameterValue("dataHeaderRegexp", "Reporter ID\\tCluster\\tGene sym.*");
			request.setParameterValue("reporterIdColumnMapping", "\\Reporter ID\\");
			request.setParameterValue("nameColumnMapping", "\\Reporter name\\");
			request.setParameterValue("symbolColumnMapping", "\\Gene sym.\\");
			request.setParameterValue("extendedColumnMapping.clusterId", "\\Cluster\\");
			request.setParameterValue("extendedColumnMapping.sequence", "\\5\\");
			request.setParameterValue("extendedColumnMapping.vector", "\\6\\");
			request.setParameterValue("extendedColumnMapping.tissue", "\\7\\");
			request.setParameterValue("extendedColumnMapping.library", "\\8\\");
			request.setParameterValue("extendedColumnMapping.accession", "\\9\\");
			request.setParameterValue("extendedColumnMapping.nid", "\\10\\");
			request.setParameterValue("extendedColumnMapping.chromosome", "\\11\\");
			request.setParameterValue("extendedColumnMapping.cytoband", "\\12\\");
			request.setParameterValue("extendedColumnMapping.markers", "\\13\\");
			request.setParameterValue("extendedColumnMapping.antibiotics", "\\14\\");
			request.setParameterValue("extendedColumnMapping.locusLink", "\\15\\");
			request.setParameterValue("minDataColumns", 20);
			request.setParameterValue("maxDataColumns", 20);
			if (reporterTypeId != 0)
			{
				request.setParameterValue("reporterType", ReporterType.getById(dc, reporterTypeId));
			}
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(pc);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = pc.getId();
			write("--Create configuration for reporter import OK");
		}
		catch (Throwable ex)
		{
			write("--Create configuration for reporter import FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_job(int pluginConfigurationId, int fileId, String mode)
	{
		if (pluginConfigurationId == 0 || fileId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.JOB)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			PluginConfiguration pc = PluginConfiguration.getById(dc, pluginConfigurationId);
			Job j = pc.newJob(null);
			j.setName(pc.getName());
			
			PluginConfigurationRequest request = j.configure(new GuiContext(Item.REPORTER, GuiContext.Type.LIST));
			write_request_information(request.getRequestInformation());
			request.setParameterValue("file", File.getById(dc, fileId));
			request.setParameterValue("mode", mode);
			request.setParameterValue("invalidUseOfNullError", "skip");
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for reporter import (mode="+mode+") OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for reporter import (mode="+mode+") FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}


	static void write_request_information(RequestInformation ri)
	{
		if (TestUtil.getSilent()) return;

		write("Request information for command: "+ri.getCommand());
		write("\tTitle:\t"+ri.getTitle());
		write("\tDescription:\t"+ri.getDescription());
		for (PluginParameter<?> pp : ri.getParameters())
		{
			ParameterType pt = pp.getParameterType();
			System.out.println("\tParameter:\t"+pp.getName() + "\t" + pp.getLabel() + "\t" + pt );
		}
	}
}

