/*
	$Id: TestUserDefaultSetting.java 5060 2009-08-19 07:02:11Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

public class TestUserDefaultSetting
{

	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing user default setting");
		write_header();
		// Standard tests: create, load, list
		int userId = TestUser.test_create(true);
		int id = test_create(userId, "fontsize", "9");
		test_duplicate(id);
		test_load(id);
		test_list(userId, -1);

		// Extra tests:

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		TestUser.test_delete(userId);
		write("++Testing user default setting "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int userId, String name, String value)
	{
		if (userId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User user = User.getById(dc, userId);
			UserDefaultSetting uds = user.newDefaultSetting(name, value);
			dc.saveItem(uds);
			dc.commit();
			id = uds.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(uds, false);
			write_item(0, uds);
			write("--Create user default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Create user default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			UserDefaultSetting uds = UserDefaultSetting.getById(dc, id);
			write_item(0, uds);
			write("--Load user default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Load user default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int userId, int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User user = User.getById(dc, userId);
			ItemResultList<UserDefaultSetting> l = UserDefaultSetting.getQuery(user).list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List user default settings OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List user default settings FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			UserDefaultSetting uds = UserDefaultSetting.getById(dc, id);
			dc.deleteItem(uds);
			dc.commit();
			write("--Delete user default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Delete user default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName\tValue\tUser");
			write("-- \t-- \t----\t-----\t----");
		}
	}
	static void write_item(int i, UserDefaultSetting uds)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+uds.getId()+"\t"+uds.getName()+"\t"+
			uds.getValue()+"\t"+uds.getUser());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_duplicate(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			UserDefaultSetting uds = UserDefaultSetting.getById(dc, id);
			User user = uds.getUser();
			UserDefaultSetting duplicate = user.newDefaultSetting(uds.getName(), uds.getValue());
			dc.saveItem(duplicate);
			dc.commit();
			dc.deleteItem(duplicate);
			dc.commit();
			throw new BaseException("Expected DatabaseException");
		}
		catch (DatabaseException ex)
		{
			write("--Duplicate user default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Duplicate user default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}


