<%-- $Id: index.jsp 5060 2009-08-19 07:02:11Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Hkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Quantity"
	import="net.sf.basedb.core.Unit"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.util.RemovableUtil"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="java.util.Arrays"
	import="java.util.HashSet"
	import="java.util.List"
	import="java.util.Collections"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%!
	private static final ItemContext defaultContext = Base.createDefaultContext("name", "name,displaySymbol");
	private static final Item itemType = Item.UNIT;
%>
<%
final int quantityId = Values.getInt(request.getParameter("quantity_id"));

final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String cmd = request.getParameter("cmd");
final String root = request.getContextPath()+"/";
final String mode = request.getParameter("mode");
final String callback = request.getParameter("callback");
final String itemId = request.getParameter("item_id");
final String listPage = "../index.jsp?ID="+ID+"&cmd=ViewItem&item_id="+quantityId;
final String viewPage = "view_unit.jsp?ID="+ID+"&quantity_id="+quantityId;
final String editPage = "edit_unit.jsp?ID="+ID+"&quantity_id="+quantityId;

String forward = null;
String redirect = null;
String message = null;
DbControl dc = null;

try
{
	if (cmd == null || "List".equals(cmd))
	{
		// Display the list page without updatinging the current context
		Base.getAndSetCurrentContext(sc, itemType, null, defaultContext, true);
		redirect = listPage;
	}
	else if ("UpdateContext".equals(cmd))
	{
		// Display the list page after updating the current context from the request parameters
		Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = listPage;
	}
	else if ("LoadContext".equals(cmd))
	{
		// Display the list page after loading a saved context
		int contextId = Values.getInt(request.getParameter("context"));
		Base.loadContext(sc, contextId, defaultContext);
		redirect = listPage;
	}
	else if ("ViewItem".equals(cmd))
	{
		// Display the view page for a single item 
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		forward = viewPage;
	}
	else if ("EditItem".equals(cmd))
	{
		// Display the edit page for a single item (should be opened in a popup)
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = editPage;
	}
	else if ("NewItem".equals(cmd))
	{
		// Display the edit page for a new item (should be opened in a popup)
		if (!sc.hasPermission(Permission.CREATE, itemType))
		{
			throw new PermissionDeniedException(Permission.CREATE, itemType.toString());
		}
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		cc.setId(0);
		forward = editPage;
	}
	else if ("UpdateItem".equals(cmd))
	{
		// Update the properties on an item (will close the popup)
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, defaultContext);
		dc = sc.newDbControl();
		Unit unit = (Unit)cc.getObject("item");
		String[] symbols = Values.getString(request.getParameter("symbols")).split("[\n\r]+");
		String displaySymbol = symbols.length == 0 ? null : symbols[0];
		if (unit == null)
		{
			Quantity quantity = Quantity.getById(dc, quantityId);
			unit = Unit.getNew(dc, quantity, displaySymbol);
			message = "Unit created";
			dc.saveItem(unit);
		}
		else
		{
			dc.reattachItem(unit, false);
			unit.setDisplaySymbol(displaySymbol);
			message = "Unit updated";
		}
		unit.setName(Values.getStringOrNull(request.getParameter("name")));
		unit.setDescription(Values.getStringOrNull(request.getParameter("description")));
		unit.setSymbols(new HashSet<String>(Arrays.asList(symbols)));
		double factor = Values.getDouble(request.getParameter("referenceFactor"), 1.0);
		double offset = Values.getDouble(request.getParameter("referenceOffset"), 0.0);
		unit.changeReferenceFactorAndOffset(factor, offset);
		dc.commit();
		cc.removeObject("item");
	}
	else if ("DeleteItem".equals(cmd))
	{
		// Delete a single item and then return to the view page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		RemovableUtil.setRemoved(dc, itemType, Collections.singleton(cc.getId()), true);
		dc.commit();
		System.out.println("cmd=" + cmd + "; cc=" + cc.getId());
		redirect = viewPage;
	}
	else if ("DeleteItems".equals(cmd))
	{
		// Delete all selected items on the list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		int numTotal = cc.getSelected().size();
		int numDeleted = RemovableUtil.delete(dc, itemType, cc.getSelected());
		dc.commit();
		if (numTotal != numDeleted)
		{
			message = (numDeleted == 0 ? "No" : "Only "+numDeleted+" of "+numTotal) + " items could be deleted, because you have no DELETE permission";
		}
		redirect = listPage+(message != null ? "&popmessage="+HTML.urlEncode(message) : "");
	}
	else if ("RestoreItem".equals(cmd))
	{
		// Restore a single item and then return to the view page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		RemovableUtil.setRemoved(dc, itemType, Collections.singleton(cc.getId()), false);
		dc.commit();
		redirect = viewPage;
	}
	else if ("RestoreItems".equals(cmd))
	{
		// Restore all selected items on the list page
		dc = sc.newDbControl();
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		int numTotal = cc.getSelected().size();
		int numRemoved = RemovableUtil.setRemoved(dc, itemType, cc.getSelected(), false);
		dc.commit();
		if (numTotal != numRemoved)
		{
			message = (numRemoved == 0 ? "No" : "Only "+numRemoved+" of "+numTotal) + " items could be restored, because you have no WRITE permission";
		}
		redirect = listPage+(message != null ? "&popmessage="+HTML.urlEncode(message) : "");
	}
	else if ("ExportItems".equals(cmd))
	{
		// Run an export plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		dc = sc.newDbControl();
		final Quantity quantity = Quantity.getById(dc, quantityId);
		dc.close();
		final ItemQuery<Unit> query = quantity.getUnits();
		cc.configureQuery(query, true);
		cc.setQuery(query);
		redirect = "../../../common/export/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&title=Export+units";
	}
	else if ("ExportItem".equals(cmd))
	{
		// Run an export plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../../common/export/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&title=Export+unit";
	}
	else if ("ImportItems".equals(cmd))
	{
		// Run an import plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		dc = sc.newDbControl();
		final Quantity quantity = Quantity.getById(dc, quantityId);
		dc.close();
		final ItemQuery<Unit> query = quantity.getUnits();
		cc.configureQuery(query, true);
		cc.setQuery(query);
		redirect = "../../../common/import/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&title=Import+units";
	}
	else if ("ImportItem".equals(cmd))
	{
		// Run an import plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../../common/import/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&title=Import+unit";
	}
	else if ("RunListPlugin".equals(cmd))
	{
		// Run another plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		dc = sc.newDbControl();
		final Quantity quantity = Quantity.getById(dc, quantityId);
		dc.close();
		final ItemQuery<Unit> query = quantity.getUnits();
		cc.configureQuery(query, true);
		cc.setQuery(query);
		redirect = "../../../common/plugin/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=LIST&main_type=OTHER&title=Run+plugin";
	}
	else if ("RunPlugin".equals(cmd))
	{
		// Run another plugin in single-item context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, pageContext, defaultContext);
		redirect = "../../../common/plugin/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&context_type=ITEM&main_type=OTHER&title=Run+plugin";
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}

}
finally
{
	if (dc != null) dc.close();
}
if (forward != null)
{
	pageContext.forward(forward);
}
else if (redirect != null)
{
	response.sendRedirect(redirect);
}
else if (message == null)
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&wait=0");
}
else
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&message="+HTML.urlEncode(message));
}

%>
